/*******************************************************************************
 * Copyright (c) 2008 The University of York.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Dimitrios Kolovos - initial API and implementation
 ******************************************************************************/
package org.eclipse.epsilon.evl.dt.editor;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IStorage;
import org.eclipse.epsilon.common.dt.editor.outline.ModuleElementLabelProvider;
import org.eclipse.epsilon.common.module.IModule;
import org.eclipse.epsilon.eol.dt.editor.EolEditor;
import org.eclipse.epsilon.evl.EvlModule;
import org.eclipse.epsilon.evl.dt.editor.outline.EvlModuleElementLabelProvider;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.part.FileEditorInput;
import org.modelbus.traceino.core.eclipse.editor.api.StringStorage;
import org.modelbus.traceino.core.eclipse.editor.api.StringStorageEditorInput;

public class EvlEditor extends EolEditor{
	
	public EvlEditor() {
		this.addTemplateContributor(new EvlEditorStaticTemplateContributor());
	}
	
	@Override
	public List<String> getKeywords() {
		
		List<String> keywords = new ArrayList<String>();
		
		keywords.add("context");
		keywords.add("constraint");
		keywords.add("guard");
		keywords.add("pre");
		keywords.add("post");
		keywords.add("assumes");
		keywords.add("critique");
		keywords.add("message");
		keywords.add("title");
		keywords.add("do");
		keywords.add("check");
		keywords.add("fix");
		keywords.add("typeOf");
		keywords.add("kindOf");
		keywords.add("high");
		keywords.add("medium");
		keywords.add("low");

		keywords.addAll(super.getKeywords());
		
		return keywords;
	}
	
	@Override
	public List<String> getBuiltinVariables() {
		
		ArrayList<String> builtIn = new ArrayList<String>();
		
		builtIn.add("constraintTrace");
		
		builtIn.addAll(super.getBuiltinVariables());
		
		return builtIn;
	}

	@Override
	public ModuleElementLabelProvider createModuleElementLabelProvider() {
		return new EvlModuleElementLabelProvider();
	}
	
	@Override
	public IModule createModule(){
		return new EvlModule();
	}
	
	//ADDED MEN
	@Override
	public void parseModule() {
		String code = null;
		
		final IModule module = createModule();
		
		final IEditorInput input = this.getEditorInput();
		
		if(input instanceof FileEditorInput){
			FileEditorInput fileInputEditor = (FileEditorInput) getEditorInput();
			IFile file = fileInputEditor.getFile();
			
			final IDocument doc = this.getDocumentProvider().getDocument(
					this.getEditorInput());
			
			// Replace tabs with spaces to match
			// column numbers produced by the parser
			code = doc.get();
			
			code = code.replaceAll("\t", " ");
			
			try {
				module.reset();
				module.parse(code, new File(file.getLocation().toOSString()));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		else if(input instanceof StringStorageEditorInput){
			try{
				final IStorage storage = ((StringStorageEditorInput) input).getStorage();
				final InputStream stream = ((StringStorage)storage).getContents();
				
				final BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(stream));
				StringBuilder stringBuilder = new StringBuilder();
				String line = null;
	
				while ((line = bufferedReader.readLine()) != null) {
					stringBuilder.append(line + "\n");
				}
	
				bufferedReader.close();
				code = stringBuilder.toString();
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
			code = code.replaceAll("\t", " ");
			
			try {
				module.reset();
				module.parse(code);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		// column numbers produced by the parser
		/*code = code.replaceAll("\t", " ");
		
		try {
			module.reset();
			module.parse(code, new File(file.getLocation().toOSString()));
		} catch (Exception e) {
			
		}*/
		
		// Update problem markers
		// Delete all the old markers and add new
		/*try {
			file.deleteMarkers(AbstractModuleEditor.PROBLEMMARKER, true, IResource.DEPTH_INFINITE);
			//file.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
			for (ParseProblem problem : module.getParseProblems()) {
				Map<String, Object> attr = new HashMap<String, Object>();
				attr.put(IMarker.LINE_NUMBER, new Integer(problem.getLine()));
				attr.put(IMarker.MESSAGE, problem.getReason());				
				int markerSeverity;
				if (problem.getSeverity() == ParseProblem.ERROR) {
					markerSeverity = IMarker.SEVERITY_ERROR;
				}
				else {
					markerSeverity = IMarker.SEVERITY_WARNING;
				}
				attr.put(IMarker.SEVERITY, markerSeverity);
				MarkerUtilities.createMarker(file, attr, AbstractModuleEditor.PROBLEMMARKER);
//				MarkerUtilities.createMarker(file, attr, IMarker.PROBLEM);
			}
		} catch (CoreException e1) {}*/
		
		if (module != null && module.getParseProblems().size() == 0) {
			notifyModuleParsedListeners(module);
		}
		
	}
	
}
